/* -*- mode: c ; c-file-style: "canonware-c-style" -*-
 ****************************************************************************
 *
 * Copyright (C) 1996-1999 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************
 *
 * Version: s19990912a
 *
 * <<< Description >>>
 *
 * Verify that libstash works, even if libstash_init() is not called.
 *
 ****************************************************************************/

#include <libstash/libstash_r.h>

void
do_mem(void)
{
  void * p;

  p = _cw_malloc(1024);
  _cw_check_ptr(p);

  p = _cw_realloc(p, 2048);
  _cw_check_ptr(p);

  _cw_free(p);
  
  p = _cw_calloc(8, 128);
  _cw_check_ptr(p);
  
  _cw_free(p);
}

void
do_dbg(void)
{
  dbg_register(cw_g_dbg, "foo");
  dbg_unregister(cw_g_dbg, "foo");
  dbg_is_registered(cw_g_dbg, "foo");
}

int
main()
{
  out_put(cw_g_out, "Test begin\n");
  do_mem();
  do_dbg();
  
  out_put(cw_g_out, "libstash_init()\n");
  libstash_init();
  do_mem();
  do_dbg();

  out_put(cw_g_out, "libstash_shutdown()\n");
  libstash_shutdown();
  do_mem();
  do_dbg();
  
  out_put(cw_g_out, "Test end\n");  
  return 0;
}
